/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 *  epix.h -- Automatically created on Sat Dec  1 22:09:13 UTC 2007  *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <iostream>
#include <vector>
#include <sstream>
#include <cmath>
#include <cstdio>
#include <cstdarg>

/* 
 * globals.h -- Top-level header file
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.0
 * Last Change: August 24, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *  This file provides the epix class to manage otherwise global 
 *  variables:
 *
 *  - dash_length (2 -- 200 true points; parameter same as dot_sep)
 *  - dash_fill   (0.05 -- 0.95, fraction of path filled when dashed)
 *  - dot_size    (0.5  --  256 true pt)
 *  - dot_sep
 *
 *  - gray_depth  (0=white -- 1=black)
 *  - angle_units (radians, degrees, revolutions)
 *  - path_style  (SOLID, DASHED, DOTTED)
 *
 *  - clipping
 *  - cropping
 *  - fill_paths
 */

#define EPIX_GLOBALS


namespace ePiX {
}

namespace ePiX {

  enum epix_path_style  {SOLID, DASHED, DOTTED};
  //  enum epix_output_type {EEPIC}; // METAPOST, SVG...

  // for backward compatibility, not in class epix below
  extern double x_min, x_max, x_size, y_min, y_max, y_size;
  extern double tix;

  inline double min(double a, double b) { return a<b ? a : b; }
  inline double max(double a, double b) { return a<b ? b : a; }

  inline double snip_to(double var, const double arg1, const double arg2)  
    {
      if      (var < min(arg1,arg2)) var = min(arg1,arg2);
      else if (var > max(arg1,arg2)) var = max(arg1,arg2);
      return var;
    }

  // global "style" constants
  class epix {
  public:
    static bool clipping;
    static bool cropping;
    static bool fill_paths;

    static bool using_pstricks;
    static std::string fillcolor; // no effect unless using_pstricks

    // functions for setting style parameters
    friend void dash_fill(double t=0.5)    { dashfill = snip_to(t,0.05,0.95); }
    friend void dash_length(double len=12) { separation = snip_to(len,2,200); }

    friend void dot_sep(double len=12)     { separation = snip_to(len,2,200); }
    friend void dot_size(double diam=3)    { dotsize = snip_to(diam,0.5,256); }

    friend void gray(double depth=0.3) { gray_depth = snip_to(depth, 0, 1); }

    friend void radians(void)         { angle_units = 1.0; }
    friend void degrees(void)         { angle_units = M_PI/180; }
    friend void revolutions(void)     { angle_units = 2*M_PI; }

    // radians per angle unit
    friend double angle(const double t) { return t*angle_units; }

    // store label rotation angle internally in degrees
    friend void label_angle(double t); // defined in functions.cc

    // set state variable
    friend void solid(void)  { PATH_STYLE = SOLID; }
    friend void dashed(double t=0) 
      { 
	PATH_STYLE = DASHED; 
	if ( t != 0) dash_fill(t); // by default, leave
      }
    friend void dotted(double t=0) 
      { 
	PATH_STYLE = DOTTED; 
	if (t != 0) dot_size(t);
      }

    friend void clip(bool arg = true)   { clipping = arg; }
    friend void crop(bool arg = true)   { cropping = arg; }
    friend void fill(bool arg = true)   { fill_paths = arg; }
    friend void use_pstricks(bool arg = true)   { using_pstricks = arg; }

    friend void begin(void); // defined in output.cc

    // for internal use
    static double get_dashfill(void)   { return dashfill; }
    static double get_dashlength(void) { return separation; }

    static double get_gray(void)       { return gray_depth; }
    static double get_dotsize(void)    { return dotsize; }
    static double full_turn(void)      { return 2*M_PI/angle_units; }
    static double get_labelangle(void) { return labelangle; } // in degrees
    static epix_path_style path_style(void) { return PATH_STYLE; }

  private:
    static double dashfill;    // 0.05 --   0.95
    static double separation;  // 2    -- 200 (true pt)
    static double gray_depth;  // 0    --   1
    static double dotsize;     // 0.5  --  10

    static double angle_units;
    static double labelangle;  // rotate labels this amount in degrees

    //    static epix_output_type OUTPUT_TYPE;
    //    static std::string font;

    static epix_path_style PATH_STYLE;
    static int begin_count; // number of times begin() has been called

  }; // end of class epix

  // Constants not meant to be modifiable except at compile time

  // Precision-related constants:
  // ePiX's idea of too small/large
  const double EPIX_EPSILON=0.0001;  // 10^-4
  const double EPIX_INFTY=100000.0;  // 10^5

  // In calculus plotting, the interval between adjacent points is too large
  // for accuracy. Divide each such interval into this many subintervals
  const int EPIX_ITERATIONS=200;


  // Output-formatting constants:
  const int EPIX_NUM_PTS=80;     // # of points in ellipses, splines
  const int EPIX_FILE_WIDTH=70;  // Width of output file
  const int EPIX_PATH_LENGTH=60; // Number of points per path segment

  // Sizes of markers et. al.
  const double EPIX_ARROWHEAD_WIDTH=1.5; // Half-width of arrowheads in pt
  const double EPIX_ARROWHEAD_RATIO=5.5; // 2*Length/width ratio of arrowheads

  // Enumeration types:
  enum epix_mark_type {PATH, CIRC, SPOT, RING, DOT, DDOT, PLUS, OPLUS, 
		       TIMES, OTIMES, DIAMOND, UP, DOWN, BOX, BBOX, 
		       H_TICK, V_TICK, TEXT};

  enum epix_label_posn {none, c, r, tr, rt, t, tl, lt, l, bl, lb, b, br, rb};

  enum epix_tick_type {TICK_NULL, H_AXIS, V_AXIS};

  enum epix_field_type {SLOPE, DART, VECTOR};

  // Deprecated constants

  const int EPIX_PAIRS_PER_LINE=4;/* Number of pairs per line in output file */


  // <pic_unit> is one of the following valid LaTeX length units: 
  // bp, cm, in, mm, pc, pt (default), or sp.
  // <pic_size> is a double that (with pic_unit) specifies unitlength
  extern double pic_size; // e.g., 1
  extern char *pic_unit;  // e.g., "pt" 


  // Shape error types...
  enum constructor_error_type {MALFORMED, MULTIPLICITY, COLLINEAR_PTS};
  enum join_error_type {TANGENT, PARALLEL, COINCIDENT, NON_COPLANAR,
			SEPARATED, CONCENTRIC};

  // and handlers
  struct constructor_error {
    constructor_error_type type;
    constructor_error(constructor_error_type x) { type = x; }
  };

  struct join_error {
    join_error_type type;
    join_error(join_error_type x) { type = x; }
  };

} /* end of namespace */

/* 
 *  triples.h -- ePiX::P class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc16
 * Last Change: August 24, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The P class (ordered triple of <double>s) and operators
 *
 *    - P(), P() (default to origin), E_1, E_2, E_3 (standard basis)
 *    - Vector space operators
 *    - Other vector operators, operands (a,b,c), (x,y,z)
 *       | dot product                      ax + by + cz
 *       norm(p) = sqrt(p|p)
 *       & componentwise product            (ax, by, cz)
 *       * cross product                    (bz-cy, cx-az, ay-bx)
 *       J rotate (x1,x2,0)-plane 1/4 turn  (-b,a,0)
 *       % orthogonalization                u%v = u - ((u|v)/(v|v))*v
 */


#define EPIX_TRIPLES


namespace ePiX {

  class P 
    {
    private:
      double X1, X2, X3;

    public:
      P(double arg1=0, double arg2=0, double arg3=0)
	: X1(arg1), X2(arg2), X3(arg3) {}

      double x1(void) const { return X1; }
      double x2(void) const { return X2; }
      double x3(void) const { return X3; }

     // increment operators
      P& operator += (const P arg)
	{
	  X1 += arg.X1; X2 += arg.X2; X3 += arg.X3;
	  return (*this);
	}

      P& operator -= (const P arg)
	{
	  X1 -= arg.X1; X2 -= arg.X2; X3 -= arg.X3;  
	  return (*this);
	}

      P& operator *= (const double c) // scalar multipication
	{
	  X1 *= c; X2 *= c; X3 *= c;
	  return (*this);
	}

      // cross product
      P& operator *= (const P v);

      // componentwise product
      P& operator &= (const P v)
	{
	  X1 *= v.X1;
	  X2 *= v.X2;
	  X3 *= v.X3;

	  return *this;
	}

      // orthogonalization u %= v: vector of the form u-k*v, perp to v
      P& operator %= (const P v);

    }; // end of class P

  typedef P triple; // compatibility alias

  const P E_1 = P(1,0,0);
  const P E_2 = P(0,1,0);
  const P E_3 = P(0,0,1);

  // vector space operations
  P operator- (const P u);  // unary negation
  P operator+ (const P u, const P v);
  P operator- (const P u, const P v);
  // scalar multiplication
  P operator* (const double c, const P v);

  inline P midpoint(const P u, const P v, const double t=0.5)
    {
      return u + t*(v-u);
    }

  // cross product
  P operator* (const P u, const P v);
  P J(const P arg); // quarter turn about E_3-axis

  // dot product
  double operator | (const P u, const P v);
  inline double norm(const P u) { return sqrt(u|u); }

  // componentwise product (a,b,c)&(x,y,z)=(ax,by,cz)
  P operator& (const P u, const P v);
  // orthogonalization
  P operator% (const P u, const P v);

  // (in)equality
  bool operator == (const P u, const P v);
  bool operator != (const P u, const P v);

} /* end of namespace */

/*
 * functions.h  -- ePiX auxiliary functions
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.0
 * Last Change: September 04, 2004
 *
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define EPIX_FUNCTIONS



namespace ePiX {

  // ePiX:: trig functions are sensitive to angle units
  double cos(double);
  double sin(double);
  double tan(double); 

  // non-mathematical behavior (i.e. vanishing:) at poles
  double sec(double);
  double csc(double);
  double cot(double);

  // and inverses
  double acos(double);
  double asin(double);
  double atan(double);

  // more sanely-named aliases
  inline double Cos(double t) { return ePiX::cos(t); }
  inline double Sin(double t) { return ePiX::sin(t); }
  inline double Tan(double t) { return ePiX::tan(t); }

  inline double Sec(double t) { return ePiX::sec(t); }
  inline double Csc(double t) { return ePiX::csc(t); }
  inline double Cot(double t) { return ePiX::cot(t); }

  inline double Acos(double t)  { return ePiX::acos(t); }
  inline double Asin(double t)  { return ePiX::asin(t); }
  inline double Atan(double t)  { return ePiX::atan(t); }

  // identity and zero functions, coordinate projections
  template<class T>T id(T arg) { return arg; }
  template<class T>double zero(T arg) { return 0; }
  template<class T>double zero(T arg1, T arg2) { return 0; }

  template<class T1, class T2>T1 proj1(T1 arg1, T2 arg2) { return arg1; }
  template<class T1, class T2>T2 proj2(T1 arg1, T2 arg2) { return arg2; }

  // P constructors
  inline P xyz(double x, double y, double z=0) 
    {
      return P(x, y, z);
    }

  inline P cyl(double r, double t, double z) 
    {
      return P(r*ePiX::cos(t), r*ePiX::sin(t), z);
    }

  inline P sph(double r, double t, double phi) 
    {
      return P(r*(Cos(t))*(Cos(phi)), r*(Sin(t))*(Cos(phi)), r*(Sin(phi)));
    }

  inline P cylindrical(P arg)
    {
      return cyl(arg.x1(), arg.x2(), arg.x3());
    }

  inline P spherical(P arg)
    {
      return sph(arg.x1(), arg.x2(), arg.x3());
    }

  inline P polar(double r, double t) { return cyl(r, t, 0); }
  inline P cis(double t) { return cyl(1, t, 0); }


  double clip_to_unit(double t);
  // utility functions with discontinuities removed
  double recip (double);
  double sinx (double);
  double sgn (double);

  // period-2 extension of absolute value on [-1,1]: \/\/\/
  double cb (double);

  int gcd (int, int);

  double inf (double f(double), double, double);
  double sup (double f(double), double, double);


  // constant functions
  class Const {
  private:
    double value;

  public:
    Const(double val=0) { value = val; }
    double operator() (double t) { return value; }
  }; // end of class Const

  // derivative and integral operators for functions of one variable
  template<class T> T deriv(T f(double), double t, double dt=EPIX_EPSILON)
  {
    return (1.0/dt)*(f(t+0.5*dt) - f(t-0.5*dt));
  }

  class D {
  private:
    double (*f)(double);
    double dt;

  public:
    D(double func(double), const double eps=EPIX_EPSILON) 
      { f = func; dt = eps; }

    P operator() (const P arg) // for plotting
      { 
	double t=arg.x1();
	return P(t, deriv(f, t, dt), 0);
      }

    // numerical values
    double eval(const double t)
      {
	return deriv(f, t, dt);
      }

    // one-sided derivatives
    double right(const double t)
      {
	return  (2.0/dt)*(f(t+0.5*dt) - f(t));
      }

    double left(const double t)
      {
	return (2.0/dt)*(f(t) - f(t-0.5*dt));
      }

  }; // end of class D


  class I {
  private:
    double (*f)(double);
    double x0; // lower limit

  public:
    I(double func(double), double a=0)
      { f = func; x0 = a; }

    P operator() (const P arg);

    double eval(const double t);

  }; // end of class I

  double newton (double f(double), double g(double), double start);
  inline double newton (double f(double), double start)
    {
      return newton(f, zero, start);
    }

} /* end of namespace */

/* 
 * domain.h -- ePiX mesh and domain classes
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc13
 * Last Change: July 12, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define EPIX_DOMAIN



namespace ePiX {

  class mesh {
  private:
    int mesh1;
    int mesh2;
    int mesh3;

  public:
    mesh(int n1=1, int n2=1, int n3=1)
      {
	mesh1 = (int)max(1, fabs(n1));
	mesh2 = (int)max(1, fabs(n2));
	mesh3 = (int)max(1, fabs(n3));
      }

    int n1(void) { return mesh1; }
    int n2(void) { return mesh2; }
    int n3(void) { return mesh3; }

  }; // end of class mesh

  typedef mesh Net; // compatibility

  // A domain is a gridded coordinate box with specified corners
  // and coarse/fine meshes, used for plotting. The "coarse" step
  // sizes are step1, etc., and the "fine" step sizes are dx1, etc.
  // Steps may be negative.
  //
  // A domain may be "sliced" by setting one of its coordinates to
  // a specified constant.
  //
  // Data members are public for convenience; each member ensures
  // its own well-constructedness.
  //
  class domain {
  public:
    P corner1;
    P corner2;
    mesh coarse;
    mesh fine;

    domain(P arg1=P(1,1,1), P arg2=P(0,0,0), 
	   mesh c=mesh(1,1,1), mesh f=mesh(1,1,1)) 
      : corner1(arg1), corner2(arg2)
      {
	int c1=c.n1(), c2=c.n2(), c3=c.n3();
	int f1=f.n1(), f2=f.n2(), f3=f.n3();

	if (corner1.x1() == corner2.x1()) { c1 = f1 = 1; }
	if (corner1.x2() == corner2.x2()) { c2 = f2 = 1; }
	if (corner1.x3() == corner2.x3()) { c3 = f3 = 1; }

	coarse = mesh(c1,c2,c3);
	fine   = mesh(f1,f2,f3);
      }

    domain(double t_min, double t_max, int num_pts)
      : corner1(t_min), corner2(t_max), coarse(num_pts), fine(num_pts) {}

    // big and small steps
    double step1(void) { return (corner2.x1() - corner1.x1())/coarse.n1(); }
    double step2(void) { return (corner2.x2() - corner1.x2())/coarse.n2(); }
    double step3(void) { return (corner2.x3() - corner1.x3())/coarse.n3(); }

    double dx1(void) { return (corner2.x1() - corner1.x1())/fine.n1(); }
    double dx2(void) { return (corner2.x2() - corner1.x2())/fine.n2(); }
    double dx3(void) { return (corner2.x3() - corner1.x3())/fine.n3(); }

    // resizing attempts to preserve real resolution
    domain& resize1(double a1, double b1);
    domain& resize2(double a2, double b2);
    domain& resize3(double a3, double b3);

    // coordinate slices
    // "snip_to" is defined in globals.h and performs the "obvious"
    // truncation: snip_to(x, a, b) = x, min(a,b), or max(a,b)
    domain slice1(double a1)
      {
	a1 = snip_to(a1, corner1.x1(), corner2.x1());

	return domain(P(a1,  corner1.x2(), corner1.x3()),
		      P(a1,  corner2.x2(), corner2.x3()),
		      mesh(1, coarse.n2(),  coarse.n3()),
		      mesh(1,   fine.n2(),    fine.n3()));
      }

    domain slice2(double a2)
      {
	a2 = snip_to(a2, corner1.x2(), corner2.x2());

	return domain(P(corner1.x1(),  a2, corner1.x3()),
		      P(corner2.x1(),  a2, corner2.x3()),
		      mesh(coarse.n1(), 1,  coarse.n3()),
		      mesh(  fine.n1(), 1,    fine.n3()));
      }

    domain slice3(double a3)
      {
	a3 = snip_to(a3, corner1.x3(), corner2.x3());

	return domain(P(corner1.x1(),  corner1.x2(), a3),
		      P(corner2.x1(),  corner2.x2(), a3),
		      mesh(coarse.n1(), coarse.n2(),  1),
		      mesh(  fine.n1(),   fine.n2(),  1));
      }

    std::vector<domain> slices1(void);
    std::vector<domain> slices2(void);
    std::vector<domain> slices3(void);

  }; // end of class domain

} // end of namespace

/*
 * map.h -- classes of map for function plotting
 *
 * Andrew D. Hwang            <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 *
 * Version 0.8.11rc13
 * Last Change: July 13, 2004
 *
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * This file provides classes that represent mappings R -> R^3 or R x R -> R^3,
 * both P-valued and assembled from double-valued components. Each class
 * provides an evaluation operator that takes 1, 2, or 3 doubles and returns
 * a P. Stubs for general coordinate systems are included.
 *
 *  - path_map:            R -> R^3
 *  - surface_map:     R x R -> R^3
 *  - space_map:   R x R x R -> R^3
 *
 *  - column_1var: assemble     R -> R^3 from components
 *  - column_2var: assemble R x R -> R^3 from components
 *
 * These classes are intended primarily for internal use, allowing the
 * plot algorithms to be templated. In new code, maps should be defined
 * as P-valued, rather than assembled from components; the "column" classes
 * are provided solely for backward compatibility.
 */

#define EPIX_MAP


namespace ePiX {

  // R -> R^3
  class path_map {
  private:
    P (*y)(double);
    P (*Y)(P);

  public:
    path_map(P f(double), P F(P) = id)
      {
	y = f;
	Y = F;
      }
    P operator() (const P arg) { return Y(y(arg.x1())); }

  }; // end of class path_map

  // R x R -> R^3
  class surface_map {
  private:
    P (*y)(double, double);
    P (*Y)(P);

  public:
    surface_map(P f(double, double), P F(P) = id)
      {
	y = f;
	Y = F;
      }
    P operator() (const P arg) { return Y(y(arg.x1(), arg.x2())); }

  }; // end of class surface_map

  // R x R x R -> R^3
  class space_map {
  private:
    P (*y)(double, double, double);
    P (*Y)(P);

  public:
    space_map(P f(double, double, double), P F(P) = id)
      {
	y = f;
	Y = F;
      }
    P operator() (const P arg) { return Y(y(arg.x1(), arg.x2(), arg.x3())); }

  }; // end of class space_map


  // assemble R -> R^3 from components
  class column_1var {
  private:
    double (*y1)(double);
    double (*y2)(double);
    double (*y3)(double);
    P (*Y)(P);

  public:
    column_1var(double f1(double), double f2(double), double f3(double) = zero,
		P F(P) = id)
      {
	y1 = f1;
	y2 = f2;
	y3 = f3;
	Y  = F;
      }
    // parameterized path from f:R -> R
    column_1var(double f(double), P F(P) = id)
      {
	y1 = id;
	y2 = f;
	y3 = zero;
	Y  = F;
      }
    P operator() (const P arg) 
      { 
	double t = arg.x1();
	return Y(P(y1(t), y2(t), y3(t))); 
      }
  }; // end of class column_1var

  // assemble R x R -> R^3 from components
  class column_2var {
  private:
    double (*y1)(double, double);
    double (*y2)(double, double);
    double (*y3)(double, double);
    P (*Y)(P);

  public:
    column_2var(double f1(double, double), 
		double f2(double, double), 
		double f3(double, double),
		P F(P) = id)
      {
	y1 = f1;
	y2 = f2;
	y3 = f3;
	Y  = F;
      }
    // omitted third component defaults to zero
    column_2var(double f1(double, double), 
		double f2(double, double), 
		P F(P) = id)
      {
	y1 = f1;
	y2 = f2;
	y3 = zero;
	Y  = F;
      }
    // parameterized surface from f:R x R -> R
    column_2var(double f(double, double), P F(P) = id)
      {
	y1 = proj1;
	y2 = proj2;
	y3 = f;
	Y  = F;
      }
    P operator() (const P arg) 
      { 
	double u = arg.x1();
	double v = arg.x2();
	return Y(P(y1(u, v), y2(u, v), y3(u, v))); 
      }
  }; // end of class column_2var

} /* end of namespace */

/* 
 * pairs.h -- ePiX pair:: class and mathematical operators
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc11
 * Last Change: June 26, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define EPIX_PAIRS



namespace ePiX {

  class pair 
    {
    private:
      double X1;
      double X2;

    public:
      pair(const double arg1=0, const double arg2=0) : X1(arg1), X2(arg2) {}

      double x1(void) const { return X1; }
      double x2(void) const { return X2; }

      pair(const P arg) 
	{ 
	  X1 = arg.x1();
	  X2 = arg.x2(); 
	}

      // unary increment operators
      pair& operator += (const pair arg)
	{
	  X1 += arg.X1;
	  X2 += arg.X2;
  
	  return (*this);
	}

      pair& operator -= (const pair arg)
	{
	  X1 -= arg.X1;
	  X2 -= arg.X2;
  
	  return (*this);
	}

      pair& operator *= (const double c)
	{
	  X1 *= c;
	  X2 *= c;
  
	  return (*this);
	}

      // complex multiplication and division
      pair& operator *= (const pair arg)
	{
	  double temp = X1;
	  X1 = temp * arg.X1 - X2 * arg.X2;
	  X2 = temp * arg.X2 + X2 * arg.X1;
  
	  return (*this);
	}
      pair& operator /= (const pair arg)
	{
	  double denom = hypot(arg.X1, arg.X2);

	  double temp = X1;
	  X1 = (temp * arg.X1 + X2 * arg.X2)/denom;
	  X2 = (X2 * arg.X1 - temp * arg.X2)/denom;
  
	  return (*this);
	}

      bool operator== (const pair u)
	{
	  return ( (X1 == u.X1) && (X2 == u.X2) );
	}

    }; // end of class pair
  
  inline pair& operator- (pair u)
    {
      return u *= -1;
    }

  inline pair operator+ (pair u, const pair v) { return u += v; }
  inline pair operator- (pair u, const pair v) { return u -= v; }

  // complex arithmetic
  inline pair& J(pair p) { return p *= pair(0,1); }
  inline pair& operator* (pair u, const pair v) { return u *= v; }
  inline pair& operator/ (pair u, const pair v) { return u /= v; }

  // dot product
  inline double operator| (const pair u, const pair v)
  {
    return u.x1()*v.x1() + u.x2()*v.x2();
  }
  inline double norm (const pair u) { return sqrt(u|u); }

  // componentwise product (a,b)&(x,y)=(ax,by)
  inline pair operator& (const pair u, const pair v)
  {
    return pair(u.x1()*v.x1(), u.x2()*v.x2());
  }

} // end of namespace

/* 
 * camera.h -- Vectors and operations
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc9
 * Last Change: June 23, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * In geometry, a "frame" is a right-handed orthonormal basis, that is,
 * an ordered set of three mututally perpendicular unit vectors, oriented
 * according to the right-hand rule. A frame has nothing to do with
 * picture frames.
 *
 * ePiX uses a camera metaphor to view points in R^3. The camera consists
 * of:
 *   - a frame ({sea, sky, eye}, with the intuitive meanings:)
 *   - a point to look at (the "target")
 *   - the distance from the camera to the target
 *   - a projection mapping from R^3 to R^2 (the "lens")
 *
 * While specific behavior is determined by the lens, the idea is that the
 * viewer sits at given <distance> from the <target>, in the direction of
 * the <eye>. The vectors <sea> and <sky> point horizontally to the right
 * and vertically upward on the screen. A <distance> of zero is taken to
 * mean "infinitely far away"; for example, point projection (<shadow>)
 * becomes orthogonal projection along the <eye> axis. Again, a specific
 * lens can override this behavior, for example by ignoring the distance.
 *
 *
 * This file provides:
 *   - the frame class, and routines for rotating a frame about the axes
 *     determined by its elements. (Note that there are no methods for
 *     rotating about fixed coordinate axes, and that rotations in R^3 do
 *     not generally commute.
 *
 *   - the camera class and routines:
 *       void rotate_sea(double); // change camera orientation
 *       void rotate_sky(double); // about respective axes
 *       void rotate_eye(double);
 *
 *       void range(double d); // set distance from camera to origin
 *       void look_at(P arg); // set target (center of field of view)
 *       void lens(pair (*screen_projection(P))); // set projection mapping
 *
 *       pair shoot(P arg); // project a point to the screen ("take a photo")
 *
 */

#define EPIX_CAMERA


namespace ePiX {

  class frame {
  private:
    // orthonormal triple
    P frame1;
    P frame2;
    P frame3;

  public:
    //constructors
    frame(void) { frame1 = E_1; frame2 = E_2; frame3 = E_3; }

    // Gram-Schmidt
    frame(P arg1, P arg2, P arg3);

    // frame elements
    P sea() { return frame1; }
    P sky() { return frame2; }
    P eye() { return frame3; }

    // rotations about frame elements
    frame rotate_frame1(double angle)
      {
	P temp2 = frame2;
	P temp3 = frame3;

	frame2 = 
	  (ePiX::cos(angle)*(temp2)) - (ePiX::sin(angle)*(temp3));
	frame3 = 
	  (ePiX::sin(angle)*(temp2)) + (ePiX::cos(angle)*(temp3));

	return *this;
      }


    frame rotate_frame2(double angle)
      {
	P temp3 = frame3;
	P temp1 = frame1;

	frame3 = 
	  (ePiX::cos(angle)*(temp3)) - (ePiX::sin(angle)*(temp1));

	frame1 = 
	  (ePiX::sin(angle)*(temp3)) + (ePiX::cos(angle)*(temp1));

	return *this;
      }

    frame rotate_frame3(double angle)
      {
	P temp1 = frame1;
	P temp2 = frame2;

	frame1 = 
	  (ePiX::cos(angle)*(temp1)) - (ePiX::sin(angle)*(temp2));

	frame2 = 
	  (ePiX::sin(angle)*(temp1)) + (ePiX::cos(angle)*(temp2));

	return *this;
      }

  }; // end of class frame


  // screen projection mappings ("lenses")
  pair shadow(P arg);
  pair fisheye(P arg);
  pair bubble(P arg);


  class epix_camera {
  private:
    P viewpt;
    P target;
    frame  orient;
    double distance;
    pair (*screen_projection)(P);

  public:

    // defaults to orthog projection on (x,y) plane from pos z-axis
    epix_camera(void)
      { 
	viewpt = P(0,0,EPIX_INFTY);
	target = P(0,0,0);
	distance = EPIX_INFTY;
	orient = frame();

	screen_projection = shadow;
      }

    P get_viewpt(void) { return viewpt; }
    P get_target(void) { return target; }
    double get_range(void) { return distance; }

    // get frame vectors
    P sea(void) { return orient.sea(); }
    P sky(void) { return orient.sky(); }
    P eye(void) { return orient.eye(); }

    // adjust position, orientation, and target
    // pitch: rotate camera up/down
    void rotate_sea(double angle)
      { 
	orient = orient.rotate_frame1(angle); 
	viewpt = target + distance*orient.eye();
      }

    // yaw: rotate camera left/right
    void rotate_sky(double angle)
      { 
	orient = orient.rotate_frame2(angle); 
      	viewpt = target + distance*orient.eye();
      }

    // roll: rotate camera about viewing axis
    void rotate_eye(double angle)
      { orient = orient.rotate_frame3(angle); } // target unchanged

    // fix target, move viewpt radially along eye()
    void range(double d);
    // fix viewpt, move target radially along eye()
    void focus(double d);

    // fix target, set viewpt arbitrarily
    void at(P arg);
    friend void viewpoint(P arg);
    friend void viewpoint(double a1, double a2, double a3);

    // fix viewpt, set target arbitrarily
    void look_at(P arg);

    // set projection mapping
    void lens(pair proj(P)) { screen_projection = proj; }

    // project a point to the screen ("shoot a photo")
    pair operator() (const P arg) { return screen_projection(arg); }

  }; // end of class epix_camera 

  extern epix_camera camera;

} /* end of namespace */

/* 
 * cropping.h -- ePiX classes and functions for cropping paths and objects
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc14
 * Last Change: July 27, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define EPIX_CROPPING


namespace ePiX {

  class crop_mask {
  private:
    double x1_min, x1_max, x1_sz;
    double x2_min, x2_max, x2_sz;

  public:
    static crop_mask Bounding_Box;
    static crop_mask Crop_Box;
    static crop_mask Picture;
    static pair Offset;

    crop_mask(const pair arg1=pair(1,1), const pair arg2=pair(0,0))
      {
	x1_min = min(arg1.x1(), arg2.x1());
	x1_max = max(arg1.x1(), arg2.x1());
	x1_sz  = x1_max - x1_min;

	x2_min = min(arg1.x2(), arg2.x2());
	x2_max = max(arg1.x2(), arg2.x2());
	x2_sz  = x2_max - x2_min;
      }

    double x_sz(void) const { return x1_sz; }
    double y_sz(void) const { return x2_sz; }

    pair bl(void)   const { return pair(x1_min, x2_min); }
    pair tr(void)   const { return pair(x1_max, x2_max); }
    //    pair size(void) const { return pair(x1_sz,  x2_sz); }

    // translate by pair
    crop_mask operator+= (const pair arg)
      {
	x1_min += arg.x1();
	x1_max += arg.x1();

	x2_min += arg.x2();
	x2_max += arg.x2();

	return *this;
      }

    bool is_onscreen(const P arg) const;

    // Given two points (one projecting to the crop_mask, one not),
    // return the point on the segment that projects to the border.
    P seek_crop(const P, const P) const;
    friend P seek_path_end(const P in, const P out);

    // functions in global scope for compatibility
    friend void bounding_box(const P, const P);

    friend void crop_box(const P, const P);
    friend void crop_box(void); // (re)set crop box to bounding box

    friend void picture(const double, const double);
    friend void picture(const P);

    friend void offset(const P arg) { crop_mask::Offset = pair(arg); }
    friend void offset(const double hoff, const double voff) 
      { crop_mask::Offset = pair(hoff, voff); }

    friend double h_size(void) { return crop_mask::Picture.x1_sz; }
    friend double v_size(void) { return crop_mask::Picture.x2_sz; }

    friend double h_offset(void) { return crop_mask::Offset.x1(); }
    friend double v_offset(void) { return crop_mask::Offset.x2(); }

  }; // end of class crop_mask


  class enclosure {
    friend class path;
  private:
    double x1_min, x2_min, x3_min;
    double x1_max, x2_max, x3_max;

    static enclosure Clip_Box;

  public:
    enclosure(const P arg1, const P arg2);
    //    bool is_cropped(const P arg);

    // functions in global scope for compatibility
    friend void clip_box(const P arg1, const P arg2);

    bool is_clipped1(const P arg) const
      {
	double x1 = arg.x1();
	return !( (x1_min <= x1) && (x1 <= x1_max) );
      }

    bool is_clipped2(const P arg) const
      {
	double x2 = arg.x2();
	return !( (x2_min <= x2) && (x2 <= x2_max) );
      }

    bool is_clipped3(const P arg) const 
      {
	double x3 = arg.x3();
	return !( (x3_min <= x3) && (x3 <= x3_max) );
      }

    bool is_inside(const P arg) const
      {
	return !(is_clipped1(arg) || is_clipped2(arg) || is_clipped3(arg));
      }

    // Given two points, one in the enclosure, the other not, return the
    // point on the boundary.
    P seek_clip(const P in, const P out) const;
    friend P seek_path_end(const P in, const P out);

    friend bool is_in_bounds(const P arg)
      {
	return enclosure::Clip_Box.is_inside(arg);
      }

    friend void clip_to(const P arg) 
      { 
	enclosure::Clip_Box = enclosure(P(0,0,0), arg); 
      }
    friend void clip_box(const P arg) 
      { 
	enclosure::Clip_Box = enclosure(arg, -arg); 
      }

    friend double clip1_min(void) { return enclosure::Clip_Box.x1_min; }
    friend double clip2_min(void) { return enclosure::Clip_Box.x2_min; }
    friend double clip3_min(void) { return enclosure::Clip_Box.x3_min; }

    friend double clip1_max(void) { return enclosure::Clip_Box.x1_max; }
    friend double clip2_max(void) { return enclosure::Clip_Box.x2_max; }
    friend double clip3_max(void) { return enclosure::Clip_Box.x3_max; }

  }; // end of class enclosure

  inline bool is_visible(const P arg)
    {
      bool visible = true;
      if (epix::clipping && !is_in_bounds(arg))
	visible = false;

      if (epix::cropping && !crop_mask::Crop_Box.is_onscreen(arg))
	visible = false;

      return visible;
    }

} /* end of namespace */

/* 
 *  path.h -- ePiX classes for polygons and paths
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc14
 * Last Change: July 16, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 */


#define EPIX_PATHS



namespace ePiX {

  class crop_mask;
  class enclosure;
  class sphere;

  class vertex {

  private:
    P location;
    bool onscreen;
    bool in_world;

  public:
    vertex(const double a1=0, const double a2=0, const double a3=0)
      {
	location   = P(a1,a2,a3);

	onscreen = true;
	in_world = true;
      }

    vertex(const P arg)
      {
	location   = arg;

	onscreen = true;
	in_world = true;
      }

    P here(void) const      { return location; }
    bool is_onscreen(void) const { return onscreen; }
    bool is_in_world(void) const { return in_world; }

    void set_crop(const bool arg) { onscreen = !arg; }
    void set_clip(const bool arg) { in_world = !arg; }

  }; //end of class vertex


  class path {
  private:
    std::vector<vertex> vertices;

    bool closed;
    bool filled;

  public:

    path(const std::vector<vertex>& data, bool loop)
      {
	vertices = data;
	closed = loop;
	filled = epix::fill_paths;
      }

    path(int num_pts)
      {
	vertices = std::vector<vertex> (num_pts);
	closed = false;
	filled = epix::fill_paths;
      }

    // line constructors
    path(const P tail, const P head, const double expand=0);
    path(const P tail, const P head, const double expand, int num_pts);

    // finite-data paths (ellipse, spline)
    path(const P center, const P axis1, const P axis2, 
	      const double t_min, const double t_max,
	      int num_pts=EPIX_NUM_PTS);

    path(const P p1, const P p2, const P p3, 
	      int num_pts=EPIX_NUM_PTS);

    path(const P p1, const P p2, 
	      const P p3, const P p4, 
	      int num_pts=EPIX_NUM_PTS);


    path(P f(double), double t_min, double t_max, int num_pts=EPIX_NUM_PTS);
    path(double f(double), double t_min, double t_max, 
	 int num_pts=EPIX_NUM_PTS);

    // polygon/polyline with variable number of vertices
    friend path polygon(int num_pts ...);
    friend path polyline(int num_pts ...);


    // concatenate path segments
    path& operator+= (const path& vertices);
    // concatenate, reversing second sequence
    path& operator-= (const path& vertices);

    bool is_closed(void) const { return closed; }
    bool is_filled(void) const { return filled; }

    void close(const bool arg=true) { closed = arg; }
    void set_fill(const bool arg=true) { filled = arg; }

    void set_crop_all(const bool arg);
    void set_clip_all(const bool arg);

    // defined in output.cc
    void crop_to(const crop_mask& screen, const bool arg=true);
    void clip_to(const enclosure& world, const bool arg=true);
    //    void hide_by(bool hiding(const P));
    void draw(void); // defined in output.cc

    void draw(sphere, bool); // spherical plotting (front/back), in sphere.cc

  }; // end of class path


  class path_pt {

  private:
    P location;
    bool start; // start/end of path segment
    bool end;

  public:
    path_pt(const P arg, bool is_start, bool is_end)
      {
	location   = arg;

	start = is_start;
	end = is_end;
      }

    path_pt(const vertex arg, bool is_start, bool is_end)
      {
	location   = arg.here();

	start = is_start;
	end = is_end;
      }
    void unset(void) { start = end = false; }

    P here(void) const   { return location; }
    bool is_start(void) const { return start; }
    bool is_end(void) const   { return end; }

  }; //end of class path_pt

} /* end of namespace */

/* 
 * segment.h -- ePiX::segment class
 * 
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc16
 * Last Change: August 24, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The segment class (unordered pair of points),
 *   and operators:
 *    - join() (alternative constructor)
 *    - endpt1(), endpt2() (no guaranteed order)
 *    - segment += P (translate by <P>)
 *    - draw() (ePiX line)
 *    - midpoint(); (also callable on pair of Ps)
 *    - segment*segment (pt of intersection of _lines_ determined by segments)
 */

#define EPIX_SEGMENT


namespace ePiX {

  class segment
    {
    private:
      P endpt1;
      P endpt2;

    public:
      segment() {};
      segment(P p1, P p2) { endpt1 = p1; endpt2 = p2; }

      // Note: Ends of segment are not "ordered" meaningfully
      P end1() const { return endpt1; }
      P end2() const { return endpt2; }

      // translation
      segment& operator += (const P& arg)
	{
	  endpt1 += arg;
	  endpt2 += arg;

	  return *this;
	}

      P midpoint() { return 0.5*(endpt1 + endpt2); }

      // segment
      void draw();

    }; /* end of segment class */

  //  inline void draw(segment arg) { line(arg.end1(), arg.end2()); }

  // translation
  inline segment& operator + (const segment& seg, const P& arg)
    {
      segment temp=seg;
      return temp += arg;
    }

  inline segment& operator + (const P& arg, const segment& seg)
    {
      segment temp=seg;
      return temp += arg;
    }

  // intersection
  P& operator * (const segment& seg1, const segment& seg2);


  // alternative constructor
  inline segment join(P p1, P p2) { return segment(p1,p2); }

} /* end of namespace */

/* 
 * circle.h -- ePiX::circle class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc9
 * Last Change: June 23, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The circle class (center, radius, unit normal) and operators. See
 *   function declarations for list of exceptions thrown
 *
 *    - circle() (defaults to origin, unit radius, in (x1,x2,0)-plane
 *    - circle(ctr, pt) (parallel to (x1,x2,0)-plane, center ctr, through pt)
 *    - circle(pt1, pt2, pt3) (arbitrary non-collinear points)
 *    - center(), radius(), perp()
 *    - circle += P (translate by <P>)
 *    - circle *= double (scale by <double>)
 *    - draw() (ePiX ellipse)
 *    - circle*segment, circle*circle (segment def'd by pts of intersection)
 */


#define EPIX_CIRCLE


namespace ePiX {

  class circle 
    {
    private:
      P ctr;
      double rad;
      P perp_to; // *unit* normal, even if rad = 0

    public:
      // defaults to unit circle in (x1,x2,0) plane
      circle(P arg1=P(0,0,0), double arg2=1, P arg3=E_3) 
	{
	  double temp=norm(arg3);
	  if (temp < EPIX_EPSILON)
	    throw constructor_error(MALFORMED);

	  else
	    {
	      ctr = arg1;
	      rad = arg2; 
	      perp_to = (1/temp)*arg3;
	    }
	}

      circle(P ctr, P pt); // center, and point, normal = E_3
      circle(P pt1, P pt2, P pt3); // three points

      P center() const { return ctr; }
      double radius() const { return rad; }
      P perp()   const { return perp_to; }

      // translation
      circle& operator += (const P& arg)
	{
	  ctr += arg;

	  return *this;
	}

      // scaling
      circle& operator *= (const double& arg)
	{
	  rad *= arg;

	  return *this;
	}

      // circle
      void draw();

    }; /* end of circle class */

  // intersection operators
  /*
   * Throws the following exceptions:
   *   NON_COPLANAR if segment, circle are not coplanar
   *   SEPARATED if coplanar but line defined by segment misses circle
   *   TANGENT if coplanar but line defined by segment tangent to circle
   */
  segment& operator * (const segment&, const circle&);

  /*
   * Throws the following exceptions:
   *   TANGENT if circles are tangent
   *   COINCIDENT if circles are identical
   *   NOT_COPLANAR if circles not coplanar
   *   SEPARATED if disjoint, neither contained in the other
   *   CONCENTRIC if disjoint, one contained in other (not nec same ctr)
   */
  segment& operator * (const circle&, const circle&);

  inline segment operator * (const circle& arg_circle, const segment& arg_seg)
    { return arg_seg*arg_circle; }

  // affine operations: translation by a triple...
  inline circle& operator + (const P& displace, const circle& circ)
    { 
      circle temp = circ;
      return temp += displace; 
    }
  inline circle& operator + (const circle& circ, const P& displace)
    { 
      circle temp = circ;
      return temp += displace; 
    }

  // and scaling by a double
  inline circle& operator * (const double& c, const circle& circ)
    { 
      circle temp = circ;
      return temp *= c; 
    }
  inline circle& operator * (const circle& circ, const double& c)
    { 
      circle temp = circ;
      return temp *= c; 
    }

} /* end of namespace */

/* 
 * sphere.h -- ePiX::sphere class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc9
 * Last Change: June 23, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The sphere class (center, radius),
 *   and operators:
 *    - sphere += P (translate by <P>)
 *    - sphere *= double (scale about center)
 *    - draw() (ePiX line)
 *    - center();
 *    - radius();
 *    - sphere*sphere (circle of intersection)
 *
 *   Hiding test, latitude and longitudes on a sphere.
 */

#define EPIX_SPHERE


namespace ePiX {

  class sphere
    {
    private:
      P ctr;
      double rad;

    public:
      sphere(P p1=P(0,0,0), double r=1) 
	{ 
	  ctr = p1; 
	  rad = r; 
	}

      sphere(P p1, P p2) 
	{ 
	  ctr = p1; 
	  rad = norm(p2-p1); 
	}

      P center() const { return ctr; }
      double radius() const { return rad; }

      // translation
      sphere& operator += (const P& arg)
	{
	  ctr += arg;
	  return *this;
	}

      // scaling
      sphere& operator *= (const double& arg)
	{
	  rad *= arg;
	  return *this;
	}

      // sphere
      void draw();

    }; /* end of sphere class */


  // translation
  inline sphere operator + (const sphere& sph, const P& arg)
    {
      sphere temp=sph;
      return temp += arg;
    }

  inline sphere operator + (const P& arg, const sphere& sph)
    {
      sphere temp=sph;
      return temp += arg;
    }

  // scaling about center
  inline sphere operator * (const double& arg, const sphere& sph)
    {
      sphere temp=sph;
      return temp *= arg;
    }

  // intersection
  circle operator * (const sphere& sph1, const sphere& sph2);


  // alternative constructor: specify pair of antipodal points
  inline sphere poles(P p1, P p2) 
    { 
      return sphere(midpoint(p1,p2), p2); 
    }

  // utility function for hidden line removal (hidden<0 reverses value)
  bool visible_on_sphere(const P& arg, bool front = true, 
			 const sphere& S=sphere());

  // {lat,long}itude lines with hiding
  void latitude(double lat, double long_min, double long_max,
		sphere S=sphere(), frame coords=frame());

  void longitude(double lngtd, double lat_min, double lat_max,
		 sphere S=sphere(), frame coords=frame());

  void back_latitude(double lat, double long_min, double long_max,
		     sphere S=sphere(), frame coords=frame());

  void back_longitude(double lngtd, double lat_min, double lat_max,
		      sphere S=sphere(), frame coords=frame());

} /* end of namespace */

/* 
 *  plane.h -- ePiX::plane class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc14
 * Last Change: July 26, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *  This file provides:
 *
 *  The plane class (point, normal),
 *  and operators:
 *   - equality, parallelity test
 *   - plane*plane (draw line of intersection)
 *   - plane*sphere (circle of intersection)
 */

#define EPIX_PLANE


namespace ePiX {

  class plane
    {
    private:
      P pt;
      P N; // UNIT normal

    public:
      plane(P point=P(0,0,0), P normal=E_3)
	{ 
	  double temp=norm(normal);
	  if (temp < EPIX_EPSILON)
	    throw constructor_error(MALFORMED);

	  pt = point;
	  N = (1.0/temp)*normal;
	}

      plane(P p1, P p2, P p3);

      bool parallel_to (const plane& arg)
	{ return (norm((this->N)*arg.N)<EPIX_EPSILON); }

      bool operator== (const plane& arg)
	{
	  // normals parallel and arg.pt lies in *this
	  return ( (*this).parallel_to(arg) &&
		   (((this->pt) - arg.pt)|(this->N)) < EPIX_EPSILON);
	}

      // return circle of intersection
      circle operator* (const sphere& S) const;
      // draw Line
      void  operator* (const plane P1);

      // draw lines of intersection of *this with clip box faces
      void draw();

  }; /* end of plane class */

  inline circle operator* (const sphere& S, const plane& P) { return P*S; }

  inline void draw(plane arg) { arg.draw(); }

} /* end of namespace */

/*
 * lengths.h -- ePiX true and LaTeX length manipulation
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.0
 * Last Change: September 04, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * In ePiX, there are three systems of coordinates:              *
 *                                                               *
 *   Cartesian (C) -- All the user needs to know about           *
 *   Picture   (P) -- The language of the LaTeX front end        *
 *   True      (T) -- Physical location on page from lower left  *
 *                                                               *
 * The coordinate triumvirates' initials form a bad physics pun. *
 *                                                               *
 * Recognized LaTeX units are: bp, cm, in, mm, pc, pt, and sp    *
 *                                                               *
 * All objects and locations are specified in C by the user; the *
 * conversion to true coordinates is transparent. The rationale  *
 * for retaining LaTeX coordinates, rather than forcing all ePiX *
 * figures to be specified in true pt, is of course that figures *
 * are sized according to printing requirements, and often not   *
 * in terms of true pt. The reason for using true pt is that raw *
 * scaling of a LaTeX figure is not robust; label positions must *
 * be specified with true pt offsets if their locations are to   *
 * scale properly when a figure is resized.                      *
 *                                                               *
 * An ePiX figure is a rectangle, described as follows:          *
 *   Cartesian: [x_min, x_max] x [y_min, y_max]                  *
 *   Picture:   [0, h_size] x [0, v_size] + (h_offset, v_offset) *
 *   True:      Only (La)TeX knows/needs to know this            *
 *                                                               *
 * ePiX provides six scale-changing functions:                   *
 *   c2p, p2c -- Cartesian -> LaTeX and back for points          *
 *   c2s, s2c -- Cartesian -> LaTeX and back for vectors         *
 *   p2t, t2p -- LaTeX -> True and back                          *
 *                                                               *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#define EPIX_LENGTHS


namespace ePiX {

  // If less than EPIX_EPSILON, set to zero 
  double truncate(double);

  pair truncate(pair);

  // "Cartesian" = Cartesian coordinates of crop_mask::Bounding_Box
  // "picture"   = Cartesian coordinates of crop_mask::Picture

  // Affine scaling
  pair c2p(pair);  // Cartesian to picture for positions
  pair p2c(pair);  // picture to Cartesian

  pair c2s(pair); // Cartesian to picture for displacements
  pair s2c(pair); // picture to Cartesian

  double p2t(double); // picture coords to true pt
  pair p2t(pair);

  double t2p(double); // true pt to picture
  pair t2p(pair);

  // functions of <pair> in picture coords
  double true_length(pair); // length in true pt
  pair true_unit(pair);     // parallel true pt unit vector

  double arc_scale(double r);
} /* end of namespace */

/*
 * output.h -- ePiX file-formatting output routines
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc14
 * Last Change: July 16, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define EPIX_OUTPUT



namespace ePiX {

  using std::cout;

  void unitlength(const char *units);

  // Basic Sectioning Functions

  // Begin/end picture
  void begin(void);
  void end(void);
  
  inline void raw_print(const pair arg)
  {
    cout << "(" << arg.x1() << "," << arg.x2() << ")";
  }


  //  std::ostream& operator<< (std::ostream& s, const P);
  //  std::ostream& operator<< (std::ostream& s, const path&);


  void print(const P);
  void print(const pair);
  void print(const P location, const P offset);
  void print_vector(const P);

  void print_alignment(epix_label_posn POSN);
  void print_marker(epix_mark_type MARK);

  // Functions that write small pieces of a LaTeX \picture environment
  // are inlined. Aside from delimiters (braces and dollar), each command
  // preserves the LaTeX grouping level.
  inline void lbrace(void) { cout << '{'; }
  inline void rbrace(void) { cout << '}'; }
  inline void dollar(void) { cout << '$'; }
  inline void newl(void)   { cout << '\n'; }

  // common \LaTeX commands

  // print arg in braces
  template<class T>void epix_grouping(T arg)
    { cout << '{' << arg << '}'; }
  // same, in math mode
  template<class T>void epix_math_grouping(T arg)
    { cout << "{$" << arg << "$}"; }

  inline void epix_neg(void)      { cout << "\\phantom{-}"; }

  inline void epix_put(void)      { cout << "\\put"; }
  inline void epix_multiput(void) { cout << "\\multiput"; }
  inline void epix_makebox(void)  { cout << "\\makebox(0,0)"; }
  inline void epix_whiten(void)   { cout << "\\whiten"; }

  inline void start_path(void)
    {
      newl();
      if (epix::using_pstricks)
	cout << "\\psline";

      else
	cout << "\\path";
    }
  inline void end_path(void) {}

  // string-valued functions for path::draw()
  inline std::string start_path_string(void)
    {
      if (epix::using_pstricks)
	return "\n\\psline";

      else
	return "\n\\path";
    }
  inline std::string end_path_string(void) 
    {
      return "";
    }

  inline void psset(std::string arg) 
    { 
      if (epix::using_pstricks)
	cout << "\n\\psset{" << arg << "}";
    }

  inline void fill_color(std::string color="white") 
    { 
      if (epix::using_pstricks)
	{
	  epix::fillcolor=color; 
	  cout << "\n\\psset{fillcolor=" << epix::fillcolor << "}";
	}
    }

  inline void epix_rotatebox()
    {
      cout << "\\rotatebox";
      epix_grouping(epix::get_labelangle());
    }

  inline void epix_colorbox()
    {
      cout << "\\colorbox";
      if (epix::using_pstricks)
	epix_grouping(epix::fillcolor);
      else
	epix_grouping("white");
    }

  inline void epix_rule(const double x, const double y)
    {
      cout << "\\rule{" << x << "pt}{" << y << "pt}";
    }

  /*
  inline void epix_rule(const double x,  const double y,
			const double dx, const double dy)
    {
      cout << "{\\kern " << dx << "pt \\rule[" << dy << "pt]{"
	   << x << "pt}{" << y << "pt}}";
    }
  */

  inline void epix_circle(const double r_pt) // radius in pt
    {
      lbrace();
      cout << "\\circle";
      epix_grouping(t2p(r_pt)); // convert radius to picture units
      rbrace();
    }
  inline void epix_disk(const double r_pt)
    {
      lbrace();
      cout << "\\circle*";
      epix_grouping(t2p(r_pt));
      rbrace();
    }

  inline void epix_newline(void)            { cout << "\n%%"; }
  inline void epix_newline(std::string msg) { cout << "\n%% " << msg; }

  inline void end_stanza(void)
  {
    epix_newline("---");
  }

  inline void end() { cout << "\n\\end{picture}\n"; }


  // Unified comment/warning/error functions
  inline void epix_warning(std::string msg)
  {
    std::clog << "ePiX WARNING: " << msg << "\n"; // buffered output
  } 

  inline void epix_error(std::string msg)
  {
    std::cerr << "ePiX ERROR: " << msg << "\n";
    exit(1);
  } 


  //  void epix_comment(char *);
  void end_stanza(void);
  void line_break(int, int);

  // Misc Styles and output formatting
  // Output styles
  void pen(std::string);
  void pen(double);

  inline void bold(void) { cout << "\n\\thicklines"; }
  inline void plain(void)  { cout << "\n\\thinlines"; }

  // ePiX provides rgb/cmyk colors via pstcol. The following are equivalent:
  //    red();
  //    rgb(1,0,0);
  //
  // cmyk(0,1,1,0); is visually equivalent but not the same.
  //
  // Densities outside the unit interval [0,1] are truncated, e.g., 
  //    rgb(2, -1, 0.3) = rgb(1, 0, 0.3).

  // red-green-blue
  void rgb(double r, double g, double b); 
  // cyan-magenta-yellow-black
  void cmyk(double c, double m, double y, double k);

  // primary colors
  inline void red(double d=1)   { rgb(d,0,0); }
  inline void green(double d=1) { rgb(0,d,0); }
  inline void blue(double d=1)  { rgb(0,0,d); }
  inline void white(double d=1) { rgb(d,d,d); }
  inline void black(double d=1) { rgb(1-d,1-d,1-d); }

  inline void cyan(double d=1)     { cmyk(d,0,0,0); }
  inline void magenta(double d=1)  { cmyk(0,d,0,0); }
  inline void yellow(double d=1)   { cmyk(0,0,d,0); }

} /* end of namespace */

/* 
 * Label.h -- ePiX::Label class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc9
 * Last Change: June 23, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The Label class
 *     - location
 *     - offset
 *     - alignment option
 *     - type
 *     - text field
 *
 *     Label(posn, f(posn)) // numerical label at specified location
 *     move_to(P) // change basepoint
 *     offset(P)  // change offset
 *     align(epix_label_posn) // change alignment
 */


#define EPIX_LABEL


namespace ePiX {

  class Label
    {
    private:
      P the_basepoint;
      std::string the_label_text;
      epix_mark_type the_label_type;
      epix_label_posn the_alignment;
      bool masked;
      P the_offset;

    public:
      Label(P basepoint, 
	    std::string label_text, 
	    epix_mark_type label_type=TEXT,
	    epix_label_posn alignment=none, 
	    bool mask=false,
	    P offset=P(0,0))
	{
	  the_basepoint = basepoint;
	  the_label_text = label_text;
	  the_label_type = label_type;
	  the_alignment = alignment;
	  masked = mask;
	  the_offset = offset;
	}

      Label(P basepoint, double f(P),
	    epix_label_posn alignment=none, 
	    bool mask=false,
	    P offset=P(0,0));

      Label move_to(P arg) { the_basepoint = arg; return *this; }
      Label offset(P arg) { the_offset = arg; return *this; }
      Label align(epix_label_posn arg) { the_alignment=arg; return *this; }

      void draw();

    }; /* end of class */

  inline void draw(Label arg) { arg.draw(); }
} /* end of namespace */

/* 
 * objects.h -- Simple picture objects: axes, grids, markers, polygons
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc12
 * Last Change: July 01, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define EPIX_OBJECTS



namespace ePiX {

  // Picture objects

  /*
   * label -- put string constant <label_text> at Cartesian position <base>
   * translated by (offset.x1, offset.x2) true points (i.e., 3rd component
   * of <offset> is discarded).
   *
   * Accepts an optional LaTeX-style positioning argument.
   * If no offset is specified, the label is centered at the given Cartesian
   * location.
   * masklabel requires the "color" package, and places the text in 
   * a white box that masks whatever is underneath and earlier in the file.
   */
  // location given as a <P>
  void label(P base, P offset, std::string label_text);
  void label(P base, std::string label_text);
  void label(P base, P offset, std::string label_text,
	     epix_label_posn);

  void masklabel(P base, P offset, std::string label_text);
  void masklabel(P base, std::string label_text);
  void masklabel(P base, P offset, std::string label_text,
		 epix_label_posn);

  // Empty and filled LaTeX circles of diameter get_dotsize() true pt
  void circ(P); // filled white circ
  void ring(P); // unfilled circ 

  void spot(P arg);
  void dot(P arg);
  void ddot(P arg);

  void box(P arg);
  void bbox(P arg);

  void marker (P, epix_mark_type);

  // Axes and coordinate grids.

  /* 
   * Coordinate axes, specified by initial and final points, number of
   * tick marks, and (optionally) number of points (for cameras that do
   * not map lines to lines). Generally num_pts should be a multiple of n.
   * h/v_axis are identical except for style of tick marks.
   */
  void h_axis_tick(P location);
  void v_axis_tick(P location);

  //  void h_axis(P tail, P head, int n);
  //  void v_axis(P tail, P head, int n);

  // n+1 = #ticks, num_pts = #segments used to draw
  void h_axis(P tail, P head, int n, int num_pts=1);
  void v_axis(P tail, P head, int n, int num_pts=1);

  // These versions may be used with gcc 2.96 and later
  inline void h_axis(int n=x_size) { h_axis(P(x_min,0), P(x_max,0), n); }
  inline void v_axis(int n=y_size) { v_axis(P(0,y_min), P(0,y_max), n); }

  // n+1 = #ticks, num_pts = #segments used to draw
  inline void h_axis(int n, int num_pts) 
    { h_axis(P(x_min,0), P(x_max,0), n, num_pts); }
  inline void v_axis(int n, int num_pts) 
    { v_axis(P(0,y_min), P(0,y_max), n, num_pts); }

  /*
   * h_axis_labels: Draws n+1 equally-spaced axis labels between 
   *    <tail> and <head>. Automatically generates label values from
   *    Cartesian coordinates, and offsets labels in true pt.
   */

  void h_axis_labels(P tail, P head, int n, P offset);
  void v_axis_labels(P tail, P head, int n, P offset);

  void h_axis_masklabels(P tail, P head, int n, P offset);
  void v_axis_masklabels(P tail, P head, int n, P offset);

  // Axis labels with LaTeX-style alignment option

  void h_axis_labels(P tail, P head, int n, 
		     P offset, epix_label_posn POSN);
  void v_axis_labels(P tail, P head, int n, 
		     P offset, epix_label_posn POSN);

  void h_axis_masklabels(P tail, P head, int n, 
			 P offset, epix_label_posn POSN);
  void v_axis_masklabels(P tail, P head, int n, 
			 P offset, epix_label_posn POSN);

  // Axis labels with default endpoints
  inline void h_axis_labels(int n, P offset)
    { h_axis_labels(P(x_min,0), P(x_max,0), n, offset); }
  inline void h_axis_masklabels(int n, P offset)
    { h_axis_masklabels(P(x_min,0), P(x_max,0), n, offset); }

  inline void h_axis_labels(int n, P offset, epix_label_posn POSN)
    { h_axis_labels(P(x_min,0), P(x_max,0), n, offset, POSN); }
  inline void h_axis_masklabels(int n, P offset, epix_label_posn POSN)
    { h_axis_masklabels(P(x_min,0), P(x_max,0), n, offset, POSN); }


  inline void v_axis_labels(int n, P offset)
    { v_axis_labels(P(0,y_min), P(0,y_max), n, offset); }
  inline void v_axis_masklabels(int n, P offset)
    { v_axis_masklabels(P(0,y_min), P(0,y_max), n, offset); }

  inline void v_axis_labels(int n, P offset, epix_label_posn POSN)
    { v_axis_labels(P(0,y_min), P(0,y_max), n, offset, POSN); }
  inline void v_axis_masklabels(int n, P offset, epix_label_posn POSN)
    { v_axis_masklabels(P(0,y_min), P(0,y_max), n, offset, POSN); }

} /* end of namespace */

/*
 * curves.h -- Ellipses, arcs, splines
 *
 * This file is part of ePiX, a preprocessor for creating high-quality
 * line figures in LaTeX
 *
 * Version 0.8.11rc16
 * Last Change: August 22, 2004
 *
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define EPIX_CURVES


namespace ePiX {

  extern double x_min, y_min, x_max, y_max;

  // lines can be "stretched" by double parameter
  void line(const P tail, const P head, const double expand=0); 
  void line(const P tail, const P head, const double expand,
	    int num_pts);

  // "Visible" portion of the line through p1, p2
  void Line(const P tail, const P head);

  // point-slope form
  void Line(const P, const double);

  void triangle(const P, const P, const P);
  void quad(const P, const P, const P, const P);

  void draw_rect(const P, const P); // opposite corners
  void rect(const P, const P, bool solid=epix::fill_paths);
  // filled rectangle; deprecated
  void swatch(const P, const P);

  // arrows
  void arrow(const P tail, const P head, double scale=1);
  inline void dart(const P tail, const P head)
  { 
    arrow(tail, head, 0.5); 
  }
  // double-tipped
  inline void aarrow(const P tail, const P head, double scale=1)
    {
      P midpt = 0.5*(tail+head);
      arrow(midpt, tail, scale);
      arrow(midpt, head, scale);
    }

  // Algebraic curves (elliptical and circular arcs, splines)
  void ellipse(const P center, const P axis1, const P axis2,  
	       const double t_min=0, 
	       const double t_max=epix::full_turn(),
	       int num_pts=EPIX_NUM_PTS);

  // for backward compatibility
  inline void ellipse_arc(const P center, 
			  const P axis1, const P axis2,
			  const double t_min, const double t_max)
    {
      ellipse(center, axis1, axis2, t_min, t_max);
    }

  // old style "center and polyradius"
  inline void ellipse (const P center, const P radius)
    { ellipse(center, radius.x1()*E_1, radius.x2()*E_2); }
  // Standard half-ellipse functions
  void ellipse_left (const P, const P);
  void ellipse_right (const P, const P);
  void ellipse_top (const P, const P);
  void ellipse_bottom (const P, const P);

  // Circular arcs parallel to (x,y)-plane
  inline void arc(const P center, const double r, 
		  const double start,  const double finish)
  { ellipse(center, r*E_1, r*E_2, start, finish); }

  void arc_arrow (const P center, const double r, 
		  const double start, const double finish, 
		  const double scale=1);


  // Quadratic and cubic splines
  void spline (const P p1, const P p2, const P p3,
	       int num_pts=EPIX_NUM_PTS);
  void spline (const P p1, const P p2, 
	       const P p3, const P p4,
	       int num_pts=EPIX_NUM_PTS);


  // A "mesh" is an ordered pair of positive integers, and is used to
  // specify the "fineness" of a mesh. Grids, like parametric surface
  // meshes, have a "coarse mesh" -- the numbers of grid intervals in
  // each direction, and a "fine mesh" -- the numbers of points used
  // to render the grid lines. Since an ePiX camera does not always
  // map lines in object space to lines on the screen, grid lines cannot
  // generally be drawn using only two points.
  // A grid may look strange unless each component of fine is a multiple
  // of the corresponding entry of coarse, of course. :)

  // Cartesian grid of specified size, mesh, and resolution
  void grid(P arg1, P arg2, mesh coarse, mesh fine);

  // coarse = fine = (n1,n2)
  void grid(P arg1, P arg2, int n1=1, int n2=1);

  inline void grid(int n1=1, int n2=1)
    {
      grid(P(x_min, y_min), P(x_max, y_max), n1, n2);
    }

  // polar grid of specified radius, mesh (rings and sectors), and resolution
  void draw_polar_grid(double r, mesh coarse, mesh fine);
  void polar_grid(double r, mesh coarse, mesh fine);

  // polar grid with n1 rings and n2 sectors
  void polar_grid(double r, int n1, int n2);


  // fractal generation
  //
  // The basic "level-1" recursion unit is a piecewise-linear path whose 
  // segments are parallel to spokes on a wheel, labelled modulo <spokes>.
  // Recursively up to <depth>, each segment is replaced by a copy of the
  // recursion unit.
  //
  // Sample data for _/\_ standard Koch snowflake:
  // const int pre_seed[] = {6, 4, 0, 1, -1, 0};
  // pre_seed[0] = spokes, pre_seed[1] = seed_length;

  void fractal (triple p, triple q, int depth, const int *pre_seed);

} /* end of namespace */

/*
 * plots.h -- parametrized plotting algorithms
 *
 * Andrew D. Hwang            <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 *
 * Version 0.8.11rc16
 * Last Change: August 24, 2004
 *
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * This file provides templated plotting algorithms and common instantiations.
 */

#define EPIX_PLOTS


namespace ePiX {

  // can't pass domain as const reference...
  template<class T> void plot(T map, domain R)
  {
    P curr;
    int i_max = (R.dx1() > 0) ? R.coarse.n1() : 0; // max summation index
    int j_max = (R.dx2() > 0) ? R.coarse.n2() : 0;
    int k_max = (R.dx3() > 0) ? R.coarse.n3() : 0;


    if (R.dx1() > 0) // there's something to draw
      {
	int count1 = R.fine.n1(); // number of intervals in subdivision
	std::vector<vertex> data1(1+count1);

	double st1 = R.dx1();
	double st2 = R.step2();
	double st3 = R.step3();

	for (int j=0; j <= j_max; ++j)
	  for (int k=0; k <= k_max; ++k)
	    {
	      curr = R.corner1 + (j*st2)*E_2 + (k*st3)*E_3;

	      for (int i=0; i <= count1; ++i, curr += st1*E_1)
		data1.at(i) = vertex(map(curr));

	      // close path if initial and terminal points coincide
	      path path1 = path(data1, false);
	      path1.draw();
	    }
      }

    if (R.dx2() > 0)
      {
	int count2 = R.fine.n2();
	std::vector<vertex> data2(1+count2);

	double st1 = R.step1();
	double st2 = R.dx2();
	double st3 = R.step3();

	for (int i=0; i <= i_max; ++i)
	  for (int k=0; k <= k_max; ++k)
	    {
	      curr = R.corner1 + (i*st1)*E_1 + (k*st3)*E_3;
	      
	      for (int j=0; j <= count2; ++j, curr += st2*E_2)
		data2.at(j) = vertex(map(curr));

	      // close path if initial and terminal points coincide
	      path path2 = path(data2, false);
	      path2.draw();
	    }
      }

    if (R.dx3() > 0)
      {
	int count3 = R.fine.n3();
	std::vector<vertex> data3(1+count3);

	double st1 = R.step1();
	double st2 = R.step2();
	double st3 = R.dx3();

	for (int i=0; i <= i_max; ++i)
	  for (int j=0; j <= j_max; ++j)
	    {
	      curr = R.corner1 + (i*st1)*E_1 + (j*st2)*E_2;

	      for (int k=0; k <= count3; ++k, curr += st3*E_3)
		data3.at(k) = vertex(map(curr));

	      // close path if initial and terminal points coincide
	      path path3 = path(data3, false);
	      path3.draw();
	    }
      }

  }; // end of (path) plot

  // plot over a list of domains
  template<class T> void plot(T map, std::vector<domain> R_list)
  {
    for (unsigned int i=0; i < R_list.size(); ++i)
      plot(map, R_list.at(i));
  }

  // paths
  template<class T>void plot (T f, double t_min, double t_max, int num_pts)
    {
      plot(f, domain(t_min, t_max, num_pts));
    }

  // f:R -> R^3
  inline void plot (P f(double), double t_min, double t_max, int num_pts)
    {
      plot(path_map(f), domain(t_min, t_max, num_pts));
    }

  // f:R -> R x R x R
  inline void plot (double f1(double), double f2(double), double f3(double),
		    double t_min, double t_max, int num_pts)
    {
      plot(column_1var(f1,f2,f3), domain(t_min, t_max, num_pts));
    }

  // f:R -> R
  inline void plot (double f(double), double t_min, double t_max, int num_pts)
    { 
      plot(id, f, zero, t_min, t_max, num_pts); 
    }

  // f: R -> R x R
  inline void plot (double f1(double), double f2(double), 
		    double t_min, double t_max, int num_pts)
    { 
      plot(f1, f2, zero, t_min, t_max, num_pts); 
    }

  inline void polarplot (double f(double), double t_min, double t_max, 
			 int num_pts)
    {
      plot(column_1var(f, id, zero, cylindrical), 
	   domain(t_min, t_max, num_pts));
    }
  
  // surfaces
  // f:R^2 -> R x R x R
  inline void plot(double f1(double u1, double u2),
		   double f2(double u1, double u2),
		   double f3(double u1, double u2), 
		   P min, P max, mesh coarse, mesh fine)
    {
      column_2var map(f1,f2,f3);
      domain dom=domain(min, max, coarse, fine);
      plot(map, dom);
    }
  inline void plot(double f1(double u1, double u2),
		   double f2(double u1, double u2),
		   double f3(double u1, double u2), 
		   domain R)
    {
      column_2var map(f1,f2,f3);
      plot(map, R);
    }

  // f:R^2 -> R
  inline void plot(double f(double u1, double u2),
		   P p1, P p2, mesh coarse, mesh fine)
    {
      plot(proj1, proj2, f, p1, p2, coarse, fine);
    }
  inline void plot(double f(double u1, double u2), domain R)
    {
      plot(column_2var(proj1, proj2, f), R);
    }

  // f:R^2 -> R^3
  inline void plot(P f(double u1, double u2),
		   P p1, P p2, mesh coarse, mesh fine)
    {
      surface_map map(f);
      domain R=domain(p1, p2, coarse, fine);
      plot(map, R);
    }
  inline void plot(P f(double u1, double u2), domain R)
    {
      surface_map map(f);
      plot(map, R);
    }


  // Derivatives and integrals
  inline void plot_deriv(double f(double t), double a, double b, int num_pts)
  { plot(D(f), a, b, num_pts); }

  inline void plot_int(double f(double), double a, double b, int num_pts)
  { plot(I(f, a), a, b, num_pts); }
  inline void plot_int(double f(double), double x0, double a, double b, 
		       int num_pts)
  { plot(I(f, x0), a, b, num_pts); }

  void tan_line(double f1(double t), double f2(double t), double t0);
  void tan_line(double f(double t), double t0);
  void tan_line(P f(double t), double t0);

  void envelope(double f1(double), double f2(double), double, double, int);
  void envelope(double f(double), double, double, int);
  void envelope(P f(double), double, double, int);

  void tan_field     (double f1(double), double f2(double), 
		      double, double, double);
  void tan_field     (P f(double), double, double, double);


  // Slope, dart, and vector fields

  // planar fields
  void draw_field (P F(double, double),
		   P, P, int, int, epix_field_type);

  inline void slope_field(P F(double, double), 
			  P p, P q, int n1, int n2)
    { draw_field(F, p, q, n1, n2, SLOPE); }

  inline void dart_field(P F(double, double), 
			 P p, P q, int n1, int n2)
    { draw_field(F, p, q, n1, n2, DART); }

  inline void vector_field(P F(double, double), 
			   P p, P q, int n1, int n2)
    { draw_field(F, p, q, n1, n2, VECTOR); }

  // spatial fields
  void draw_field (P F(double, double, double),
		   P, P, int, int, epix_field_type);

  inline void slope_field(P F(double, double, double), 
			  P p, P q, int n1, int n2)
    { draw_field(F, p, q, n1, n2, SLOPE); }

  inline void dart_field(P F(double, double, double), 
			 P p, P q, int n1, int n2)
    { draw_field(F, p, q, n1, n2, DART); }

  inline void vector_field(P F(double, double, double), 
			   P p, P q, int n1, int n2)
    { draw_field(F, p, q, n1, n2, VECTOR); }


  // Solutions of ODE systems
  template<class VF> void euler_plot(VF field, P start, 
				     double t_min, double t_max, int num_pts)
  {
    std::vector<vertex> data(num_pts+1);

    P curr = start;
    const double dt = t_max/(num_pts*EPIX_ITERATIONS);
    const double dseek = t_min/(num_pts*EPIX_ITERATIONS);

    if (fabs(t_min/num_pts) > EPIX_EPSILON) // seek beginning of path
      for (int i=0; i <= num_pts*EPIX_ITERATIONS; ++i)
	curr += dseek*field(curr);

    //                             Euler's method ---v
    for (int i=0; i <= num_pts*EPIX_ITERATIONS; ++i, curr += dt*field(curr))
      if (i%EPIX_ITERATIONS == 0)
	data.at(i/EPIX_ITERATIONS) = vertex(curr); // N.B. integer division

    path temp(data, false);

    temp.draw();
    end_stanza();
  }


  // start at time 0
  inline void ode_plot (P F(double, double), P start, 
			double t_max, int num_pts)
    { 
      surface_map vf(F); // convert to map:P -> P
      euler_plot(vf, start, 0, t_max, num_pts);
    }

  inline void ode_plot (P F(double, double, double), P start, 
			double t_max, int num_pts)
    { 
      space_map vf(F);
      euler_plot(vf, start, 0, t_max, num_pts);
    }

  // arbitrary start time
  inline void ode_plot (P F(double, double), P start, 
			double t_min, double t_max, int num_pts)
    { 
      surface_map vf(F); // convert to map:P -> P
      euler_plot(vf, start, t_min, t_max, num_pts);
    }

  inline void ode_plot (P F(double, double, double), P start, 
			double t_min, double t_max, int num_pts)
    { 
      space_map vf(F);
      euler_plot(vf, start, t_min, t_max, num_pts);
    }


  // flow x0 under field for specified time
  template<class VF> P flow(VF field, P x0, double t_max, int num_pts=0)
  {
    P curr = x0;

    if (num_pts == 0) // use "sensible" default; hardwired constant 4
      num_pts = 4*(1 + (int)ceil(fabs(t_max)));

    const double dt = t_max/(num_pts*EPIX_ITERATIONS);

    for (int i=0; i <= num_pts*EPIX_ITERATIONS; ++i)
      curr += dt*field(curr);

    return curr;
  }

  inline P flow (P F(double, double), P start, double t_max,
		   int num_pts = 0)
  {
    surface_map vf(F);
    return flow(vf, start, t_max, num_pts);
  }

  inline P flow (P F(double, double, double), P start, double t_max,
		   int num_pts = 0)
  {
    space_map vf(F);
    return flow(vf, start, t_max, num_pts);
  }


  // Jay Belanger's shaded plot routines -- December 1, 2002

  void shadeplot(double f1(double), double f2(double), double, double, int);
  inline void shadeplot(double f(double), double t_min, double t_max, 
			int num_pts)
    {
      shadeplot(f, zero, t_min, t_max, num_pts);
    }

  inline void blackplot(double f1(double), double f2(double), 
			double t_min, double t_max, int num_pts)
    {
      double temp_gray = epix::get_gray();
      gray(1);
      shadeplot(f1, f2, t_min, t_max, num_pts);
      gray(temp_gray);
    }

  inline void blackplot(double f(double), double t_min, double t_max, 
			int num_pts)
    {
      blackplot(f, zero, t_min, t_max, num_pts);
    }

  inline void whiteplot(double f1(double), double f2(double), 
			double t_min, double t_max, int num_pts)
    {
      double temp_gray = epix::get_gray();
      gray(0);
      shadeplot(f1, f2, t_min, t_max, num_pts);
      gray(temp_gray);
    }

  inline void whiteplot(double f(double), double t_min, double t_max, 
			int num_pts)
    {
      whiteplot(f, zero, t_min, t_max, num_pts);
    }


  // Adaptive plotting
  void adplot (double f1(double), double f2(double), double, double, int);

  inline void adplot(double f(double), double t_min, double t_max, int num_pts)
  { adplot(id, f, t_min, t_max, num_pts); }

  // wiremesh surface plotting
  void plot(P Phi(double, double), P, P, mesh N, mesh num_pts);

  inline void clipplot(P Phi(double, double), P p1, P p2, 
		       mesh N, mesh num_pts)
    {
      bool temp_clip = epix::clipping;
      clip();
      plot(Phi, p1, p2, N, num_pts);
      clip(temp_clip);
    }

  inline void cropplot(P Phi(double, double), P p1, P p2, 
		       mesh N, mesh num_pts)
    {
      bool temp_crop = epix::cropping;
      crop();
      plot(Phi, p1, p2, N, num_pts);
      crop(temp_crop);
    }

} /* end of namespace */

/* 
 *  dataplot.h -- ePiX::P data plotting and operations
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc14
 * Last Change: July 24, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 */


#define EPIX_DATAPLOT


namespace ePiX {

  typedef std::vector<std::vector<double> > FILEDATA;

  // read columns of file into FILEDATA
  void read(const char*, FILEDATA&);

  // plot specified columns
  void plot(const FILEDATA& data_columns, epix_mark_type TYPE,
	    unsigned int col1=1, unsigned int col2=2, unsigned int col3=0,
	    P f(double, double, double) = xyz);

  void plot(const char* filename, epix_mark_type TYPE, int columns, 
	    unsigned int col1=1, unsigned int col2=2, unsigned int col3=0,
	    P f(double, double, double) = xyz);

  inline void data_plot(const char* filename, epix_mark_type TYPE)
    {
      epix_warning("data_plot deprecated; please use plot");
      plot(filename, TYPE, 2, 1, 2, 0);
    }

  // functions for data analysis
  double avg(FILEDATA data, unsigned int col1);
  double var(FILEDATA data, unsigned int col1);
  double covar(FILEDATA data, unsigned int col1, unsigned int col2);

  void regression(FILEDATA data, unsigned int col1=1, unsigned int col2=2);

} /* end of namespace */ 


/*
 * geometry.h 
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.11rc9
 * Last Change: June 23, 2004
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define EPIX_GEOMETRY


namespace ePiX {

  // Flag for type of projection to the sphere
  enum sphere_proj_type {RADIAL, STEREO_N, STEREO_S};

  // recommended for internal use only
  void draw_sphereplot(double f1(double), double f2(double), double f3(double),
		       double t_min, double t_max, int num_pts, bool hidden, 
		       sphere_proj_type TYPE, const sphere& S);

  void draw_sphereplot(P Phi(double), double t_min, double t_max, 
		       int num_pts, bool hidden, sphere_proj_type TYPE,
		       const sphere& S);

  // stereographic projection; path first projected vertically
  inline void frontplot_N(double f1(double), double f2(double),
			  double t_min, double t_max, int num_pts, 
			  const sphere& S = sphere())
  { draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, true, STEREO_N, S); }

  inline void backplot_N(double f1(double), double f2(double),
			 double t_min, double t_max, int num_pts,
			 const sphere& S = sphere())
  { draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, false, STEREO_N, S); }

  inline void frontplot_S(double f1(double), double f2(double),
			  double t_min, double t_max, int num_pts, 
			  const sphere& S = sphere())
  { draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, true, STEREO_S, S); }

  inline void backplot_S(double f1(double), double f2(double),
			 double t_min, double t_max, int num_pts, 
			 const sphere& S = sphere())
  { draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, false, STEREO_S, S); }


  // Radial projection from center
  inline void frontplot_R(P phi(double), double t_min, double t_max, 
			  int num_pts, const sphere& S = sphere())
  { draw_sphereplot(phi, t_min, t_max, num_pts, true, RADIAL, S); }

  inline void backplot_R(P phi(double), double t_min, double t_max, 
			 int num_pts, const sphere& S = sphere())
  { draw_sphereplot(phi, t_min, t_max, num_pts, false, RADIAL, S); }


  // Hyperbolic lines
  void hyperbolic_line (P, P);
  void disk_line (P, P);

} /* end of namespace */

